<?php

namespace App\Http\Controllers;

use App\Imports\KoleksiImport;
use App\Models\Koleksi;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Auth;
use Maatwebsite\Excel\Facades\Excel;

class KoleksiController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function siswa()
    {
        $koleksi = Koleksi::all();
        return view('koleksi.siswa', ['koleksi' => $koleksi]);
    }

    public function pustakawan()
    {
        $koleksi = Koleksi::all();
        return view('koleksi.index', ['koleksi' => $koleksi]);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $this->authorize('isAdmin', auth()->user());
        return view('koleksi.tambah');
    }

    public function search(Request $request)
    {
        if ($request->ajax()) {
            $keySearch = $request->input('key');

            $query = Koleksi::query();

            // Add your search conditions to the query
            if ($keySearch) {
                $query->where('judul', 'like', '%' . $keySearch . '%')
                    ->orWhere('pengarang', 'like', '%' . $keySearch . '%');
            }

            $query->orderBy('judul', 'asc');

            // Paginate the results
            $koleksi = $query->paginate(10);

            // $view = view('koleksi.table', compact('koleksi'))->render();
            $output = '';

            if ($koleksi->count() > 0) {
                foreach ($koleksi as $row) {
                    $output .= '<tr>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->kode . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->jenis . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->judul . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->penerbit . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center" >' . $row->tahun_terbit . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->pengarang . '</td>';
                    if (Auth::user()->is_admin) {
                        $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center" >' . $row->jumlah_buku . '</td>';
                    }
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center" >' . $row->tersedia . '</td>';
                    //Aksi
                    if (Auth::user()->is_admin) {
                        $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center relative items-center justify-evenly">';
                        $output .= '<a href="' . route('koleksi.edit', ['koleksi' => $row]) . '" class="absolute left-0">';
                        $output .= '<i class="text-xl bi-pencil-square"></i></a>';
                        $output .= '<form method="post" action="' . route('koleksi.destroy', ['koleksi' => $row]) . '">';
                        $output .= csrf_field(); // Use csrf_field() to generate the CSRF token input
                        $output .= method_field('DELETE'); // Use method_field() to generate the hidden input for DELETE method
                        $output .= '<button type="submit" class="btn btn-link"><i class="text-xl bi bi-trash-fill" style="color:red"></i></button>';
                        $output .= '</form>';
                        $output .= '</td>';
                    }
                    $output .= '</tr>';
                }
            } else {
                $output = '<tr><td colspan="9">Tidak ada hasil</td></tr>';
            }

            // Extract the pagination links
            $paginationLinks = $koleksi->links()->toHtml();

            // Return a JSON response with the view and pagination links
            return Response::json(['html' => $output, 'paginationLinks' => $paginationLinks]);
        }
    }

    public function siswaSearch(Request $request)
    {
        if ($request->ajax()) {
            $keySearch = $request->input('key');

            $query = Koleksi::query();

            // Add your search conditions to the query
            if ($keySearch) {
                $query->where('judul', 'like', '%' . $keySearch . '%')
                    ->orWhere('pengarang', 'like', '%' . $keySearch . '%');
            }

            $query->orderBy('judul', 'asc');

            // Paginate the results
            $koleksi = $query->paginate(10);

            // $view = view('koleksi.table', compact('koleksi'))->render();
            $output = '';

            if ($koleksi->count() > 0) {
                foreach ($koleksi as $row) {
                    $output .= '<tr>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->kode . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->jenis . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->judul . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->penerbit . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center" >' . $row->tahun_terbit . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->pengarang . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center" >' . $row->tersedia . '</td>';
                    $output .= '</tr>';
                }
            } else {
                $output = '<tr><td colspan="6">Tidak ada hasil</td></tr>';
            }

            // Extract the pagination links
            $paginationLinks = $koleksi->links()->toHtml();

            // Return a JSON response with the view and pagination links
            return Response::json(['html' => $output, 'paginationLinks' => $paginationLinks]);
        }
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $request->validate([
            'kode' => 'required',
            'jenis' => 'required',
            'judul' => 'required',
            'jumlah_buku' => 'required|numeric|min:1',
            'penerbit' => 'required',
            'tahun_terbit' => 'required|numeric',
            'pengarang' => 'required',
        ]);

        $data = [
            'kode' => $request['kode'],
            'jenis' => $request['jenis'],
            'judul' => $request['judul'],
            'jumlah_buku' => $request['jumlah_buku'],
            'tersedia' => $request['jumlah_buku'] - 1,
            'penerbit' => $request['penerbit'],
            'tahun_terbit' => $request['tahun_terbit'],
            'pengarang' => $request['pengarang'],
        ];

        Koleksi::create($data);

        return redirect(route('pustakawan.koleksi.index'));
    }

    /**
     * Display the specified resource.
     */
    public function show(Koleksi $koleksi)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Koleksi $koleksi)
    {
        $this->authorize('isAdmin', auth()->user());
        return view('koleksi.edit', compact('koleksi'));
    }


    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Koleksi $koleksi)
    {
        $this->authorize('isAdmin', auth()->user());

        $request->validate([
            'kode' => 'required',
            'jenis' => 'required',
            'judul' => 'required',
            'jumlah_buku' => 'required|numeric|min:1',
            'penerbit' => 'required',
            'tahun_terbit' => 'required|numeric',
            'pengarang' => 'required',
        ]);

        if ($koleksi->jumlah_buku != $request->jumlah_buku) {
            $selisih = $request->jumlah_buku - $koleksi->jumlah_buku;
            $tersedia = $koleksi->tersedia + $selisih;
        }else {
            $tersedia = $koleksi->tersedia;
        }

        $data = [
            'kode' => $request['kode'],
            'jenis' => $request['jenis'],
            'judul' => $request['judul'],
            'jumlah_buku' => $request['jumlah_buku'],
            'penerbit' => $request['penerbit'],
            'tahun_terbit' => $request['tahun_terbit'],
            'tersedia' => $tersedia,
            'pengarang' => $request['pengarang'],
        ];

        $koleksi->update($data);
        return redirect(route('pustakawan.koleksi.index'))->with('success', 'Berhasil mengubah koleksi');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Koleksi $koleksi)
    {
        $koleksi->delete();

        return redirect(route('pustakawan.koleksi.index'))->with('success', 'Berhasil menghapus koleksi');
    }

    public function import(Request $request)
    {
        $this->authorize('isAdmin', auth()->user());
        $import = new KoleksiImport();
        if (!$request->hasFile('file')) {
            return redirect()->back()->with('error', 'Tidak ada file yang diunggah.');
        }

        try {
            Excel::import($import, $request->file('file'));
        } catch (\Maatwebsite\Excel\Validators\ValidationException $e) {
            $failures = $e->failures();
            $errors = [];
            foreach ($failures as $failure) {
                $errors[] = $failure->errors();
            }
            return redirect()->back()->with('error', 'Terjadi kesalahan. Data di aplikasi mungkin ada yang sudah berubah. Periksa lagi file dan data di aplikasi sebelum mencoba lagi.');
        }

        return redirect()->route('pustakawan.koleksi.index')->with('success', 'Penambahan koleksi berhasil dilakukan!');
    }

    public function migrate()
    {
        return view('koleksi.migrate');
    }
}
