<?php

namespace App\Http\Controllers;

use App\Models\Kunjungan;
use App\Models\Siswa;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class KunjunganController extends Controller
{

    public function image()
    {
        return view('kunjungan.image');
    }
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $kunjungan = Kunjungan::with('siswa')->get();
        return view('kunjungan.index', ['kunjungan' => $kunjungan]);
    }

    public function pustakawan()
    {
        $kunjungan = Kunjungan::with('siswa')->get();
        return view('kunjungan.index', ['kunjungan' => $kunjungan]);
    }

    public function siswa()
    {
        // Retrieve the authenticated user
        $user = Auth::user();

        // Paginate the results with 10 items per page
        $kunjungan = Kunjungan::where('nisn', $user->siswa->nisn)
            ->orderBy('tanggal', 'desc')
            ->paginate(10);

        return view('kunjungan.siswa', ['kunjungan' => $kunjungan]);
    }



    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $this->authorize('isAdmin', auth()->user());

        return view('kunjungan.tambah');
    }

    public function search(Request $request)
    {

        if ($request->ajax()) {
            $query = Kunjungan::query();

            $query->with('siswa');

            if (!Auth::user()->is_admin) {
                $nisn = Auth::user()->siswa->nisn;
                $query->where('nisn', $nisn);
            }

            // Filter by name or NISN
            if (!empty($request->search)) {
                $query->where('nisn', 'like', '%' . $request->search . '%')
                    ->orWhereHas('siswa', function ($query) use ($request) {
                        $query->where('nama', 'like', '%' . $request->search . '%');
                    });
            }

            // Filter by date range
            if (!empty($request->start_date) && !empty($request->end_date)) {
                $query->whereBetween('tanggal', [$request->start_date, $request->end_date]);
            }

            // Filter by tingkat and kelas
            if (!empty($request->tingkat)) {
                $query->whereHas('siswa', function ($query) use ($request) {
                    $query->where('tingkat', $request->tingkat);
                });
            }

            if (!empty($request->kelas)) {
                $query->whereHas('siswa', function ($query) use ($request) {
                    $query->where('kelas', $request->kelas);
                });
            }

            // Sort by tanggal
            $query->orderBy('tanggal', $request->tanggal);

            $data = $query->paginate(10); // Adjust the number based on your needs

            $output = '';
            if ($data->count() > 0) {
                $pageNumber = $data->currentPage();
                $perPage = $data->perPage();
                $startIndex = ($pageNumber - 1) * $perPage + 1;

                foreach ($data as $index => $row) {
                    $output .= '<tr>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">' . ($startIndex + $index) . '</td>';
                    if (Auth::user()->is_admin) {
                        $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm">' . $row->nisn . '</td>';
                    }
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm">' . $row->siswa->nama . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">' . $row->siswa->tingkat . $row->siswa->kelas . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">' . $row->tanggal . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm">' . $row->keterangan . '</td>';
                    if (Auth::user()->is_admin) {
                        // Edit link
                        $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm"><div class="flex gap-2"><a href="' . route('kunjungan.edit', ['kunjungan' => $row]) . '"><i class="bi bi-pencil-square text-xl"></i></a>';
                        // Delete form
                        $output .= '<form method="post" action="' . route('kunjungan.destroy', ['kunjungan' => $row]) . '">';
                        $output .= csrf_field(); // Use csrf_field() to generate the CSRF token input
                        $output .= method_field('DELETE'); // Use method_field() to generate the hidden input for DELETE method
                        $output .= '<button type="submit" class="btn btn-link"><i class="bi bi-trash-fill  text-xl"  style="color:red"></i></button></div>';
                        $output .= '</form>';
                        $output .= '</td>';
                    }
                    $output .= '</tr>';
                }
            } else {
                $output = '<tr><td colspan="8">Tidak ada hasil</td></tr>';
            }

            return response()->json(['html' => $output, 'pagination' => $data->links()->toHtml()]);
        }
    }


    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $this->authorize('isAdmin', auth()->user());

        $data = $request->validate([
            'nisn' => 'required|exists:siswas,nisn',
            'tanggal' => 'required|date',
            'keterangan' => 'required|in:Meminjam,Diskusi,Belajar,Berkunjung,Lainnya',
        ]);

        Kunjungan::create($data);

        return redirect(route('pustakawan.kunjungan.index'));
    }

    /**
     * Display the specified resource.
     */
    public function show(Kunjungan $kunjungan)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Kunjungan $kunjungan)
    {
        $this->authorize('isAdmin', auth()->user());

        return view('kunjungan.edit', ['kunjungan' => $kunjungan]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Kunjungan $kunjungan)
    {
        $this->authorize('isAdmin', auth()->user());

        $data = $request->validate([
            'nisn' => 'required|exists:siswas,nisn',
            'tanggal' => 'required|date',
            'keterangan' => 'required|in:Meminjam,Diskusi,Belajar,Berkunjung,Lainnya',
        ]);

        $kunjungan->update($data);
        return redirect(route('pustakawan.kunjungan.index'))->with('success', 'Kunjungan updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Kunjungan $kunjungan)
    {
        $this->authorize('isAdmin', auth()->user());

        $kunjungan->delete();
        return redirect(route('pustakawan.kunjungan.index'))->with('success', 'Kunjungan berhasil dihapus');
    }

    public function getData(Request $request)
    {
        $this->authorize('isAdmin', auth()->user());
        $data = Siswa::where('nama', 'like', '%' . $request->searchItem . '%');
        return $data->paginate(10, ['*'], 'page', $request->page);
    }

}
