<?php

namespace App\Http\Controllers;

use App\Models\Koleksi;
use App\Models\Peminjaman;
use App\Models\Siswa;
use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;

use function PHPUnit\Framework\isNull;

class PeminjamanController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $peminjaman = Peminjaman::all();
        return view('peminjaman.index', ['peminjaman' => $peminjaman]);
    }

    public function pustakawan()
    {
        $peminjaman = Peminjaman::all();
        return view('peminjaman.index', ['peminjaman' => $peminjaman]);
    }

    public function siswa()
    {
        $peminjaman = Peminjaman::where('nisn', Auth::user()->siswa->nisn)
            ->orderBy('tanggal_peminjaman', 'desc')
            ->paginate(10);

        return view('peminjaman.siswa', ['peminjaman' => $peminjaman]);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('peminjaman.tambah');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $request->validate([
            'nisn' => 'required',
            'koleksi_id' => 'required',
        ]);

        $tanggalPeminjaman = Carbon::parse($request['tanggal_peminjaman']);
        $batasPengembalian = $tanggalPeminjaman->copy()->addDays(4)->format('Y-m-d H:i:s');
        $today = today();
        if ($batasPengembalian < $today) {
            $status = 'Perlu Dikembalikan';
        } else {
            $status = 'Dipinjam';
        }

        $data = [
            'nisn' => $request['nisn'],
            'koleksi_id' => $request['koleksi_id'],
            'tanggal_peminjaman' => $tanggalPeminjaman->format('Y-m-d H:i:s'),
            'batas_pengembalian' => $batasPengembalian,
            'status' => $status
        ];

        Peminjaman::create($data);

        Koleksi::where('id', $request['koleksi_id'])->decrement('tersedia');

        return redirect(route('pustakawan.peminjaman.index'));
    }

    /**
     * Display the specified resource.
     */
    public function show(Peminjaman $peminjaman)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Peminjaman $peminjaman)
    {
        return view('peminjaman.edit', compact('peminjaman'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Peminjaman $peminjaman)
    {
        $request->validate([
            'nisn' => 'required',
            'koleksi_id' => 'required',
            'tanggal_peminjaman' => 'required',
            'batas_pengembalian' => 'required',
        ]);

        // Kalau tanggal pengembalian dihapus, mengurangi ketersediaan buku
        if ($peminjaman->tanggal_pengembalian && !$request->tanggal_pengembalian) {
            $koleksi = Koleksi::where('id',$request->koleksi_id);
            $cekKoleksi = $koleksi->value('tersedia');
            if ($cekKoleksi < 1) {
                return redirect()->back()->with('error', 'Gagal mengubah data peminjaman. Koleksi tidak tersedia.')->withInput();
            } else {
                $koleksi->decrement('tersedia');
            }
        }

        // Buku tidak bisa diganti
        // Peminjam tidak bisa diganti

        $tglkembali = $request->tanggal_pengembalian;
        $formattedTanggalPengembalian = Carbon::parse($tglkembali);
        $formattedBatasPengembalian = Carbon::parse($request->batas_pengembalian);
        $today = today();

        if ($tglkembali) {
            $status = ($formattedTanggalPengembalian > $formattedBatasPengembalian) ? 'Terlambat' : 'Tepat Waktu';
        } else {
            if ($formattedBatasPengembalian < $today) {
                $status = 'Perlu Dikembalikan';
            } else {
                $status = 'Dipinjam';
            }
        }

        // Jika ada perubahan status menjadi terlambat, maka siswa.terlambat dikurangi
        if ($peminjaman->status != $status) {
            if ($status == 'Terlambat') {
                $peminjam = Siswa::where('nisn',$peminjaman->nisn);
                $peminjam->increment("terlambat");
            }elseif ($peminjaman->status == 'Terlambat') {
                $peminjam = Siswa::where('nisn',$peminjaman->nisn);
                $peminjam->decrement("terlambat");
            }
        }

        $data = [
            'nisn' => $request->nisn,
            'koleksi_id' => $request->koleksi_id,
            'tanggal_peminjaman' => $request->tanggal_peminjaman,
            'batas_pengembalian' => $request->batas_pengembalian,
            'tanggal_pengembalian' => $request->tanggal_pengembalian,
            'status' => $status,
        ];

        $peminjaman->update($data);
        return redirect(route('pustakawan.peminjaman.index'))->with('success', 'Berhasil mengubah data peminjaman');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Peminjaman $peminjaman)
    {
        if ($peminjaman->status == "Dipinjam" || $peminjaman->status == "Perlu Dikembalikan") {
            Koleksi::where('id', $peminjaman['koleksi_id'])->increment('tersedia');
        }
        if ($peminjaman->status == "Terlambat") {
            Siswa::where('nisn', $peminjaman['nisn'])->decrement('terlambat');
        }
        $peminjaman->delete();
        return redirect(route('pustakawan.peminjaman.index'))->with('success', 'Berhasil menghapus peminjaman');
    }


    /**
     * Show the form perpanjangan
     */

    public function perpanjang(Request $request, Peminjaman $peminjaman)
    {
        return view('peminjaman.extend', compact('peminjaman'));
    }

    /**
     * Perpanjang masa peminjaman buku
     */

    public function extend(Request $request, Peminjaman $peminjaman)
    {
        $hari = $request->tambahan;
        $today = today();

        $batasPengembalian = Carbon::parse($peminjaman->batas_pengembalian)->addDays($hari);

        if ($batasPengembalian < $today) {
            $status = 'Perlu Dikembalikan';
        } else {
            $status = 'Dipinjam';
        }

        $peminjaman->update([
            'batas_pengembalian' => $batasPengembalian,
            'status' => $status,
        ]);

        return redirect(route('pustakawan.peminjaman.index'))->with('success', 'Berhasil menambah masa peminjaman');
    }

    /**
     * Mengembalikan buku yang dipinjam
     */
    public function return(Request $request, Peminjaman $peminjaman)
    {
        $tanggalPengembalian = now();
        $formattedTanggalPengembalian = Carbon::parse($tanggalPengembalian);
        $formattedBatasPengembalian = Carbon::parse($peminjaman->batas_pengembalian)->addDay();

        // Compare both dates directly
        if (($formattedTanggalPengembalian > $formattedBatasPengembalian)) {
            $status = 'Terlambat';
            Siswa::where('nisn', $peminjaman['nisn'])->increment('terlambat');
        } else {
            $status = 'Tepat Waktu';
        }

        $peminjaman->update([
            'tanggal_pengembalian' => $tanggalPengembalian,
            'status' => $status,
        ]);

        Koleksi::where('id', $peminjaman['koleksi_id'])->increment('tersedia');

        return redirect(route('pustakawan.peminjaman.index'))->with('success', 'Berhasil melakukan pengembalian');
    }

    public function getSiswa(Request $request)
    {
        $data = Siswa::where('nama', 'like', '%' . $request->searchItem . '%');
        return $data->paginate(10, ['*'], 'page', $request->page);
    }

    public function getKoleksi(Request $request)
    {
        $data = Koleksi::where('judul', 'like', '%' . $request->searchItem . '%')->where('tersedia', '>', 0);
        return $data->paginate(10, ['*'], 'page', $request->page);
    }

    public function getTerlambat(Request $request)
    {
        $nisn = $request->input('nisn');
        $terlambat = Siswa::where('nisn', $nisn)->value('terlambat');

        return response()->json(['terlambat' => $terlambat]);
    }

    public function getPeminjamanData(Request $request)
    {
        $nisn = $request->input('nisn');
        $status = $request->input('status');
        $statusCount = Peminjaman::where('nisn', $nisn)->where('status', $status)->count();

        return response()->json(['status' => $statusCount]);
    }

    public function search(Request $request)
    {
        if ($request->ajax()) {
            $key = $request->input('search');

            $query = Peminjaman::query();

            $query->with('siswa');
            $query->with('koleksi');

            // Filter by peminjam or Judul
            if (!empty($key)) {
                $query->where(function ($query) use ($key) {
                    $query->whereHas('siswa', function ($query) use ($key) {
                        $query->where('nama', 'like', '%' . $key . '%');
                    })
                        ->orWhereHas('koleksi', function ($query) use ($key) {
                            $query->where('judul', 'like', '%' . $key . '%');
                        });
                });
            }


            // Filter by date range
            if (!empty($request->start_date) && !empty($request->end_date)) {
                $query->whereBetween($request->kategori, [$request->start_date, $request->end_date]);
            }

            // Filter by Status
            if (!empty($request->status)) {
                if ($request->status == 'sudah') {
                    $query->whereNotNull('tanggal_pengembalian');
                } elseif ($request->status == 'dipinjam') {
                    $status = 'Dipinjam';
                    $query->where('status', $status);
                } elseif ($request->status == 'semua') {
                    $status = '';
                    $query->where('status', 'like', '%' . $status . '%');
                } elseif ($request->status == 'perlu') {
                    $status = 'Perlu Dikembalikan';
                    $query->where('status', $status);
                } else {
                    $status = 'Terlambat';
                    $query->where('status', $status);
                }
            }

            $query->orderByRaw(
                "
                CASE
                    WHEN status = 'Perlu Dikembalikan' THEN 1
                    WHEN status = 'Dipinjam' THEN 2
                    ELSE 3
                END
            "
            )->orderBy('tanggal_peminjaman', 'desc');

            $data = $query->paginate(10); // Adjust the number based on your needs

            $output = '';
            if ($data->count() > 0) {
                foreach ($data as $row) {
                    $output .= '<tr>';
                    // Peminjam
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm">' . $row->siswa->nama . '</td>';
                    // Judul Koleksi
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm">' . $row->koleksi->judul . '</td>';

                    // Pengembalian / Perpanjang
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">';
                    $output .= '<a title="Kembalikan"';
                    if (is_null($row->tanggal_pengembalian)) {
                        $output .= 'href="' . route('peminjaman.return', ['peminjaman' => $row]) . '">';
                    }
                    $output .= '    <i class="bi bi-arrow-return-left"></i></a>';
                    $output .= ' / ';
                    $output .= '<a title="Perpanjang Peminjaman"';
                    if (is_null($row->tanggal_pengembalian)) {
                        $output .= 'href="' . route('peminjaman.perpanjang', ['peminjaman' => $row]) . '">';
                    }
                    $output .= '    <i class="bi bi-plus-square"></i></a>';
                    $output .= '</td>';

                    // Tanggal Pinjam
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">' . $row->tanggal_peminjaman . '</td>';
                    // Batas Pinjam
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">' . $row->batas_pengembalian . '</td>';
                    // Tanggal Kembali
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">'; { {
                            is_null($row->tanggal_pengembalian) ?
                                $output .= '-' :
                                $output .= $row->tanggal_pengembalian;
                        }
                    }
                    $output .= '</td>';
                    // Status
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center">' . $row->status . '</td>';
                    // Aksi
                    if (Auth::user()->is_admin) {
                        $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-center relative items-center justify-evenly"><div class="flex gap-2">';
                        // Edit
                        $output .= '<a title="Edit" href="' . route('peminjaman.edit', ['peminjaman' => $row]) . '">';
                        $output .= '<i class="bi bi-pencil-square text-xl"></i></a>';
                        // Hapus
                        $output .= '<form method="post" action="' . route('peminjaman.destroy', ['peminjaman' => $row]) . '">';
                        $output .= csrf_field(); // Use csrf_field() to generate the CSRF token input
                        $output .= method_field('DELETE'); // Use method_field() to generate the hidden input for DELETE method
                        $output .= '<button type="submit" class="btn btn-link"><i class="text-xl bi bi-trash-fill"  style="color:red"></i></button>';
                        $output .= '</form></div>';

                        $output .= '</td>';
                    }
                    $output .= '</tr>';
                }
            } else {
                $output = '<tr><td colspan="8">Tidak ada hasil</td></tr>';
            }

            // Extract the pagination links
            $paginationLinks = $data->links()->toHtml();

            // Return a JSON response with the view and pagination links
            return response()->json(['html' => $output, 'pagination' => $paginationLinks]);
        }
    }
}
