<?php

namespace App\Http\Controllers;

use App\Imports\SiswaImport;
use App\Models\Siswa;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Maatwebsite\Excel\Facades\Excel;


class SiswaController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $siswa = Siswa::all();
        return view('siswa.index', ['siswaRecords' => $siswa]);
    }

    public function pustakawan()
    {
        $siswa = Siswa::all();
        return view('siswa.index', ['siswaRecords' => $siswa]);
    }

    public function siswa()
    {
        $siswa = Siswa::all();
        return view('siswa.index', ['siswaRecords' => $siswa]);
    }

    public function search(Request $request)
    {
        if ($request->ajax()) {
            $query = Siswa::query();

            // Filter by name or NISN
            if (!empty($request->search)) {
                $query->where('nisn', 'like', '%' . $request->search . '%')
                    ->orWhere('nama', 'like', '%' . $request->search . '%');
            }

            // Filter by tingkat and kelas
            if (!empty($request->tingkat)) {
                $query->where('tingkat', $request->tingkat);
            }

            if (!empty($request->kelas)) {
                $query->where('kelas', $request->kelas);
            }

            $query->orderBy('nama');

            $data = $query->paginate(10);

            $output = '';
            if ($data->count() > 0) {
                $pageNumber = $data->currentPage();
                $perPage = $data->perPage();
                $startIndex = ($pageNumber - 1) * $perPage + 1;

                foreach ($data as $index => $row) {
                    $output .= '<tr>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm">' . ($startIndex + $index) . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->nisn . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->nis . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->nama . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->tingkat . $row->kelas . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->tempat_lahir . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->tanggal_lahir . '</td>';
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm" >' . $row->jenis_kelamin . '</td>';
                    // Edit link
                    $output .= '<td class="px-5 py-5 border-b border-gray-200 bg-white text-sm"><div class="flex gap-2"><a href="' . route('siswa.edit', ['siswa' => $row]) . '"><i class="text-xl bi bi-pencil-square"></i></a>';                    // Delete form
                    $output .= '<form method="post" action="' . route('siswa.destroy', ['siswa' => $row]) . '">';
                    $output .= csrf_field(); // Use csrf_field() to generate the CSRF token input
                    $output .= method_field('DELETE'); // Use method_field() to generate the hidden input for DELETE method
                    $output .= '<button type="submit" class="btn btn-link"><i class="text-xl bi bi-trash-fill" style="color:red"></i></button>';
                    $output .= '</form></div>';
                    $output .= '</tr>';
                }
            } else {
                $output = '<tr><td colspan="8">Tidak ada hasil</td></tr>';
            }

            return response()->json(['html' => $output, 'pagination' => $data->links()->toHtml()]);
        }
        return redirect()->back();
    }


    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $this->authorize('isAdmin', auth()->user());
        return view('siswa.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $this->authorize('isAdmin', auth()->user());
        // Validate the request data
        $request->validate([
            'nama' => 'required',
            'nisn' => 'required|unique:siswas',
            'nis' => 'required',
            'tingkat' => 'required|numeric|between:7,9',
            'kelas' => 'required|in:A,B,C,D,E,F,G,H',
            'tgl_lahir' => 'required|date',
            'tempat_lahir' => 'required',
            'jk' => 'required|in:L,P'
        ]);

        // Create the User
        $user = User::create([
            'username' => $request->input('nisn'),
            'password' => Hash::make($request->input('nisn')),
            'is_admin' => false,
        ]);

        // Create the associated Siswa
        $siswa = Siswa::create([
            'user_id' => $user->id,
            'nisn' => $request->input('nisn'),
            'nis' => $request->input('nis'),
            'nama' => $request->input('nama'),
            'tingkat' => $request->input('tingkat'),
            'kelas' => $request->input('kelas'),
            'tempat_lahir' => $request->input('tempat_lahir'),
            'tanggal_lahir' => $request->input('tgl_lahir'),
            'jenis_kelamin' => $request->input('jk'),
        ]);

        // Update the password if a new password is provided
        $newPassword = $request->input('new_password');

        if (!empty($newPassword)) {
            $user->update([
                'password' => Hash::make($newPassword),
            ]);
        }

        return redirect(route('pustakawan.siswa.index'))->with('success', 'Siswa created successfully');
    }

    /**
     * Display the specified resource.
     */
    public function show(Siswa $siswa)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Siswa $siswa)
    {
        $this->authorize('isAdmin', auth()->user());
        return view('siswa.edit', ['siswa' => $siswa]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Siswa $siswa)
    {
        $this->authorize('isAdmin', auth()->user());
        // Validate the request data if needed
        $request->validate([
            'nama' => 'required',
            'nis' => 'required',
            'tingkat' => 'required|numeric|between:7,9',
            'kelas' => 'required|in:A,B,C,D,E,F,G,H',
            'tgl_lahir' => 'required|date',
            'tempat_lahir' => 'required',
            'jk' => 'required|in:L,P',
            'new_password' => 'nullable|min:6', // New password is optional and should be at least 6 characters
        ]);

        // Update the fields that don't depend on the password
        $siswa->update([
            'nama' => $request->input('nama'),
            'nis' => $request->input('nis'),
            'tingkat' => $request->input('tingkat'),
            'kelas' => $request->input('kelas'),
            'tanggal_lahir' => $request->input('tgl_lahir'),
            'tempat_lahir' => $request->input('tempat_lahir'),
            'jenis_kelamin' => $request->input('jk'),
        ]);

        // Update the password if a new password is provided
        $newPassword = $request->input('new_password');

        if (!empty($newPassword)) {
            $siswa->update([
                'password' => bcrypt($newPassword),
            ]);
        }

        return redirect(route('pustakawan.siswa.index'))->with('success', 'Siswa updated successfully');
    }


    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Siswa $siswa)
    {
        $this->authorize('isAdmin', auth()->user());
        // Delete related kunjungan
        $siswa->kunjungan()->delete();

        // Delete related peminjaman
        $siswa->peminjaman()->delete();

        // Delete the siswa
        $user_id = $siswa->user_id;
        $siswa->delete();

        // Delete user associated with siswa
        User::where('id', $user_id)->delete();

        return redirect(route('pustakawan.siswa.index'))->with('success','Siswa berhasil dihapus');
    }

    //TODO: not yet tested

    public function import(Request $request)
    {
        if (!$request->hasFile('file')) {
            return redirect()->back()->with('error', 'Tidak ada file yang diunggah.');
        }

        $this->authorize('isAdmin', auth()->user());
        $import = new SiswaImport();

        try {
            Excel::import($import, $request->file('file'));
        } catch (\Maatwebsite\Excel\Validators\ValidationException $e) {
            $failures = $e->failures();
            $errors = [];
            foreach ($failures as $failure) {
                $errors[] = $failure->errors();
            }
            return redirect()->back()->with('error', 'Terjadi kesalahan. Data di aplikasi mungkin ada yang sudah berubah. Periksa lagi file dan data di aplikasi sebelum mencoba lagi.');
        }

        if ($import->hasErrors()) {
            return redirect()->back()->with('error', 'Terjadi kesalahan dalam database.');
        }

        return redirect()->route('pustakawan.siswa.index')->with('success', 'Migrasi data berhasil dilakukan!');

    }

    public function migrate()
    {
        return view('siswa.migrate');
    }

    public function destroySiswaTingkat9() {
        $this->authorize('isAdmin', auth()->user());

        // Find and delete all siswa records with tingkat equal to 9
        Siswa::where('tingkat', 9)->get()->each(function ($siswa) {
            // Delete related kunjungan
            $siswa->kunjungan()->delete();

            // Delete related peminjaman
            $siswa->peminjaman()->delete();

            // Delete the siswa
            $user_id = $siswa->user_id;
            $siswa->delete();

            // Delete user associated with siswa
            User::where('id', $user_id)->delete();
        });

        return redirect(route('pustakawan.siswa.migrate'))->with('success', 'Semua siswa dengan tingkat 9 berhasil dihapus');
    }

}
