<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class UserController extends Controller
{
    //
    public function pustakawan()
    {
        return view('user.pustakawan');
    }

    public function siswa()
    {
        return view('user.siswa');
    }

    public function changePassword(Request $request)
    {
        // Retrieve the authenticated user
        $user = Auth::user();

        if ($user->is_admin) {
            $route = 'pustakawan.akun.index';
        } else {
            $route = 'siswa.akun.index';
        }

        // Validate the request with custom messages
        $request->validate([
            'old_password' => 'required',
            'new_password' => 'required|string|min:8|confirmed',
        ], [
            'old_password.required' => 'Kata sandi lama harus diisi.',
            'new_password.required' => 'Kata sandi baru harus diisi.',
            'new_password.min' => 'Kata sandi baru minimal 8 karakter.',
            'new_password.confirmed' => 'Konfirmasi kata sandi baru tidak cocok.',
        ]);


        // Check if the entered old password is correct
        if (Hash::check($request->old_password, $user->password)) {

            // Update the user's password
            $user->update([
                'password' => Hash::make($request->new_password),
            ]);

            // Determine the route based on the user's role

            // Redirect to the appropriate route with a success message
            return redirect()->route($route)->with('success', 'Berhasil mengubah kata sandi');

        }

        // Redirect back with an error message if the old password is incorrect
        return redirect()->route($route) // Replace with your actual route
        ->withInput() // Preserve form input
        ->withErrors(['old_password' => 'Kata sandi lama salah.']); // Provide an error message
    }

}
