<?php

namespace App\Imports;

use Carbon\Carbon;
use Illuminate\Support\Facades\Log;
use App\Models\User;
use App\Models\Siswa;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\ToModel;
use Illuminate\Support\Str;
use Illuminate\Database\QueryException;


class SiswaImport implements ToCollection
{
    /**
    * @param Collection $collection
    */

    public function collection(Collection $collection)
    {
        if ($collection[0][0] !== 'nisn' ||
            $collection[0][1] !== 'nis' ||
            $collection[0][2] !== 'nama' ||
            $collection[0][3] !== 'tingkat' ||
            $collection[0][4] !== 'kelas' ||
            $collection[0][5] !== 'tempat_lahir' ||
            $collection[0][6] !== 'tanggal_lahir' ||
            $collection[0][7] !== 'jenis_kelamin') {
            return;
        }

        foreach ($collection as $index => $row) {
            if ($row->filter(function ($value) { return $value !== null; })->isEmpty()) {
                continue;
            }

            if ($index === 0) continue;

            try {
                $siswa = Siswa::where('nisn', $row[0])->first();

                if ($siswa) {
                    // Siswa already exists, update the data
                    $siswa->update([
                        'tingkat' => $row[3],
                        'kelas' => $row[4]
                    ]);
                } else {
                    // Siswa does not exist, create a new one
                    $user = User::create([
                        'name' => $row[2],
                        'username' => $this->generateRandomUsername($row[2]),
                        'password' => bcrypt($row[0]),
                    ]);

                    $siswa = Siswa::create([
                        'user_id' => $user->id,
                        'nisn' => $row[0] ?? '-',
                        'nis' => $row[1] ?? '-',
                        'nama' => $row[2] ?? '-',
                        'tingkat' => $row[3] ?? '-',
                        'kelas' => $row[4] ?? '-',
                        'tempat_lahir' => $row[5] ?? '-',
                        'tanggal_lahir' => $row[6] ?? '-',
                        'jenis_kelamin' => $row[7] ?? '-'
                    ]);
                }
            } catch (QueryException $e) {
                // Handle the exception (e.g., log or return an error response)
                $this->hasErrors = true;
            }
        }
    }

    public function hasErrors()
    {
        return $this->hasErrors;
    }


        public function model(array $row)
    {
        // Find or create Siswa based on nisn
        $siswa = Siswa::updateOrCreate(['nisn' => $row['nisn']], $row);

        // If Siswa is newly created, create a corresponding User
        if (!$siswa->wasRecentlyCreated) {
            $user = User::create([
                'name' => 'siswa',
                'username' => $this->generateRandomUsername($row['nama']),
                'password' => bcrypt(Str::random(10)),
            ]);

            $siswa->user()->associate($user);
            $siswa->save();
        }

        return $siswa;
    }

    // Helper function to generate a random username based on nama
    private function generateRandomUsername($nama)
    {
        // Remove spaces from the name
        $namaWithoutSpaces = str_replace(' ', '', $nama);

        // Take the first 10 characters (or fewer if the name is shorter)
        $shortenedNama = substr($namaWithoutSpaces, 0, 10);

        // Convert the initials to uppercase
        $initials = implode('', array_map('ucfirst', str_split($shortenedNama)));

        // Append a random number between 100 and 999
        $username = $initials . rand(100, 999);

        // Check if the username already exists
        $user = User::where('username', $username)->first();
        if ($user) {
            // If the username exists, recursively call the function to generate a new one
            return $this->generateRandomUsername($nama);
        }

        return $username;
    }

}
