<?php

use App\Http\Controllers\DashboardController;
use App\Http\Controllers\KoleksiController;
use App\Http\Controllers\LaporanController;
use App\Http\Controllers\SaranController;
use App\Http\Controllers\SiswaController;
use App\Http\Controllers\UserController;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\KunjunganController;
use App\Http\Controllers\AuthController;
use App\Http\Controllers\PeminjamanController;
use App\Models\Peminjaman;

/*
|--------------------------------------------------------------------------
| Auth
|--------------------------------------------------------------------------
*/

require __DIR__ . '/auth.php';

Route::middleware('guest')->group(function () {

    Route::get('/', function () {
        return redirect('/login');
    });
});

Route::middleware(['auth', 'auth.admin'])->group(function () {

    Route::get('/pustakawan/', [DashboardController::class, 'pustakawan'])
        ->name('dashboard.pustakawan');
});

Route::middleware('auth')->group(function () {

    Route::get('/siswa/', [DashboardController::class, 'siswa'])
        ->name('dashboard.siswa');
});


/*
|--------------------------------------------------------------------------
| Kritik Saran
|--------------------------------------------------------------------------
*/
Route::middleware(['auth', 'auth.admin'])->group(function () {
    Route::get('/pustakawan/saran', [SaranController::class, 'pustakawan'])->name('pustakawan.saran.index');
    Route::delete('/saran/{saran}', [SaranController::class, 'destroy'])->name('saran.destroy');
});

Route::middleware('auth')->group(function () {
    Route::get('/siswa/saran', [SaranController::class, 'index'])->name('siswa.saran.index');
    Route::post('/siswa/saran/store', [SaranController::class, 'store'])->name('siswa.saran.store');
});


/*
|--------------------------------------------------------------------------
| Kunjungan
|--------------------------------------------------------------------------
*/

Route::middleware(['auth', 'auth.admin'])->group(function () {

    //pustakawan
    Route::get('/pustakawan/kunjungan/get', [KunjunganController::class, 'getData'])->name('kunjungan.get');
    Route::get('/pustakawan/kunjungan', [KunjunganController::class, 'pustakawan'])->name('pustakawan.kunjungan.index');
    Route::get('/pustakawan/kunjungan/tambah', [KunjunganController::class, 'create'])->name('kunjungan.tambah');
    Route::post('/pustakawan/kunjungan', [KunjunganController::class, 'store'])->name('kunjungan.store');
    Route::get('/pustakawan/kunjungan/{kunjungan}/ubah', [KunjunganController::class, 'edit'])->name('kunjungan.edit');
    Route::put('/pustakawan/kunjungan/{kunjungan}/perbarui', [KunjunganController::class, 'update'])->name('kunjungan.update');
    Route::delete('/pustakawan/kunjungan/{kunjungan}/hapus', [KunjunganController::class, 'destroy'])->name('kunjungan.destroy');
    Route::delete('/pustakawan/kunjungan/{kunjungan}/hapus', [KunjunganController::class, 'destroy'])->name('kunjungan.destroy');
    Route::get('/pustakawan/kunjungan/search', [KunjunganController::class, 'search'])->name('kunjungan.search');
});

Route::middleware('auth')->group(function () {
    //siswa
     Route::get('/siswa/kunjungan', [KunjunganController::class, 'siswa'])->name('siswa.kunjungan.index');
});

/*
|--------------------------------------------------------------------------
| Koleksi
|--------------------------------------------------------------------------
*/

Route::middleware(['auth', 'auth.admin'])->group(function () {

    //pustakawan
    Route::get('/pustakawan/koleksi', [KoleksiController::class, 'pustakawan'])->name('pustakawan.koleksi.index');
    Route::get('/pustakawan/koleksi/get', [KoleksiController::class, 'getData'])->name('koleksi.get');
    Route::get('/pustakawan/koleksi/search', [KoleksiController::class, 'search'])->name('koleksi.search');
    Route::get('/pustakawan/koleksi/tambah', [KoleksiController::class, 'create'])->name('koleksi.tambah');
    Route::post('/pustakawan/koleksi', [KoleksiController::class, 'store'])->name('koleksi.store');
    Route::get('/pustakawan/koleksi/{koleksi}/edit', [KoleksiController::class, 'edit'])->name('koleksi.edit');
    Route::put('/pustakawan/koleksi/{koleksi}/update', [KoleksiController::class, 'update'])->name('koleksi.update');
    Route::delete('/pustakawan/koleksi/{koleksi}/destroy', [KoleksiController::class, 'destroy'])->name('koleksi.destroy');
    Route::post('pustkawan/koleksi/import', [KoleksiController::class, 'import'])->name('koleksi.import');
    Route::get('/pustakawan/koleksi/migrasi', [KoleksiController::class, 'migrate'])->name('koleksi.migrate');

    //pengembalian routes
    Route::get('/pengembalian', function () {
        return view('peminjamanPengembalian.pengembalian');
    });
});

Route::middleware('auth')->group(function () {
    Route::get('/siswa/koleksi', [KoleksiController::class, 'siswa'])->name('siswa.koleksi.index');
    Route::get('/siswa/koleksi/siswaSearch', [KoleksiController::class, 'siswaSearch'])->name('koleksi.siswaSearch');
});



/*
|--------------------------------------------------------------------------
| Peminjaman
|--------------------------------------------------------------------------
*/

Route::middleware(['auth', 'auth.admin'])->group(function () {

    //pustakawan
    Route::get('/pustakawan/peminjaman/getSiswa', [PeminjamanController::class, 'getSiswa'])->name('peminjaman.getSiswa');
    Route::get('/pustakawan/peminjaman/getKoleksi', [PeminjamanController::class, 'getKoleksi'])->name('peminjaman.getKoleksi');
    Route::get('/pustakawan/peminjaman', [PeminjamanController::class, 'pustakawan'])->name('pustakawan.peminjaman.index');
    Route::get('/pustakawan/peminjaman/tambah', [PeminjamanController::class, 'create'])->name('peminjaman.tambah');
    Route::post('/pustakawan/peminjaman', [PeminjamanController::class, 'store'])->name('peminjaman.store');
    Route::get('/pustakawan/peminjaman/{peminjaman}/ubah', [PeminjamanController::class, 'edit'])->name('peminjaman.edit');
    Route::put('/pustakawan/peminjaman/{peminjaman}/perbarui', [PeminjamanController::class, 'update'])->name('peminjaman.update');
    Route::delete('/pustakawan/peminjaman/{peminjaman}/hapus', [PeminjamanController::class, 'destroy'])->name('peminjaman.destroy');
    Route::get('/pustakawan/peminjaman/{peminjaman}/perpanjang', [PeminjamanController::class, 'perpanjang'])->name('peminjaman.perpanjang');
    Route::put('/pustakawan/peminjaman/{peminjaman}/extend', [PeminjamanController::class, 'extend'])->name('peminjaman.extend');
    Route::get('/pustakawan/peminjaman/{peminjaman}/return', [PeminjamanController::class, 'return'])->name('peminjaman.return');
    Route::get('/pustakawan/peminjaman/search', [PeminjamanController::class, 'search'])->name('peminjaman.search');
    Route::get('/pustakawan/peminjaman/getTerlambat', [PeminjamanController::class, 'getTerlambat'])->name('peminjaman.getTerlambat');
    Route::get('/pustakawan/peminjaman/getPeminjamanData', [PeminjamanController::class, 'getPeminjamanData'])->name('peminjaman.getPeminjamanData');
});

Route::middleware('auth')->group(function () {
    //siswa
    Route::get('/siswa/peminjaman', [PeminjamanController::class, 'siswa'])->name('siswa.peminjaman.index');
});


/*
|--------------------------------------------------------------------------
| Laporan
|--------------------------------------------------------------------------
*/

Route::middleware(['auth', 'auth.admin'])->group(function () {

    Route::get('/pustakawan/laporan', [LaporanController::class, 'index'])->name('pustakawan.laporan.index');
    Route::get('/laporan/akun', function () {return view('Laporan/laporanAkun');});
    Route::get('/laporanPengunjung', function () {return view('Laporan/laporanPengunjung');});

    Route::get('/laporan/koleksi/lihat', [LaporanController::class, 'lihatLaporanKoleksi'])->name('laporan.koleksi.lihat');
    Route::get('/laporan/koleksi/cetak', [LaporanController::class, 'buatLaporanKoleksi'])->name('laporan.koleksi.cetak');

    Route::get('/laporan/kunjungan', function () {return view('Laporan/kunjungan/laporan');})->name('laporan.kunjungan');

    Route::get('/laporan/peminjaman', function () {return view('Laporan/peminjaman/laporan');})->name('laporan.peminjaman');

    Route::get('/laporan/akun', function () {
        return view('Laporan/akun/index');
    })->name('laporan.akun');


    Route::get('/laporan/kunjungan/lihat', [LaporanController::class, 'lihatLaporanKunjungan'])->name('laporan.kunjungan.lihat');
    Route::get('/laporan/peminjaman/lihat', [LaporanController::class, 'lihatLaporanpeminjaman'])->name('laporan.peminjaman.lihat');
    Route::get('/laporan/akun/lihat', [LaporanController::class, 'lihatLaporanAkun'])->name('laporan.akun.lihat');
    Route::get('/laporan/migrasi/lihat', [LaporanController::class, 'migrasi'])->name('laporan.migrasi.lihat');

});

/*
|--------------------------------------------------------------------------
| AKUN siswa
|--------------------------------------------------------------------------
*/

Route::middleware(['auth', 'auth.admin'])->group(function () {
    //pustakawan
    Route::get('/pustakawan/siswa', [SiswaController::class, 'pustakawan'])->name('pustakawan.siswa.index');
    Route::get('/pustakawan/siswa/tambah', [SiswaController::class, 'create'])->name('siswa.tambah');
    Route::post('/pustakawan/siswa', [SiswaController::class, 'store'])->name('siswa.store');
    Route::get('/pustakawan/siswa/migrasi', [SiswaController::class, 'migrate'])->name('pustakawan.siswa.migrate');
    Route::get('/siswa/search', [SiswaController::class, 'search'])->name('siswa.search');
    Route::post('/siswa/import', [SiswaController::class, 'import'])->name('siswa.import');
    Route::get('/siswa/{siswa}/edit', [SiswaController::class, 'edit'])->name('siswa.edit');
    Route::put('/siswa/{siswa}/update', [SiswaController::class, 'update'])->name('siswa.update');
    Route::delete('/siswa/{siswa}/destroy', [SiswaController::class, 'destroy'])->name('siswa.destroy');
    Route::post('/siswa/import', [SiswaController::class, 'import'])->name('siswa.import');
    Route::post('/siswa/destroy9', [SiswaController::class, 'destroySiswaTingkat9'])->name('siswa.destroy9');

});


/*
|--------------------------------------------------------------------------
| AKUN admin
|--------------------------------------------------------------------------
*/
Route::middleware(['auth', 'auth.admin'])->group(function () {
    Route::get('/pustakawan/akun', [UserController::class, 'pustakawan'])->name('pustakawan.akun.index');
    Route::post('/pustakawan/akun/password', [UserController::class, 'changePassword'])->name('change.password');
});

Route::middleware('auth')->group(function () {
    Route::get('/siswa/akun', [UserController::class, 'siswa'])->name('siswa.akun.index');
});

